#  File: model.py
#
#  Copyright (C) 2008 Christopher R. Gabriel <cgabriel@truelite.it>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
import os, os.path, sys
import random
try:
    import hashlib
    Hasher = hashlib.md5
except ImportError:
    import md5
    Hasher = md5.new
from datetime import datetime
from sqlobject import *
try:
    import sqlite3
    sqlite_connect = sqlite3.Connection
except ImportError:
    from pysqlite2 import dbapi2
    sqlite_connect = dbapi2.connect

class Users(SQLObject):
    username = StringCol(notNone=True)
    action = StringCol(notNone=True)
    timestamp = DateTimeCol(default=datetime.utcnow)

class Request(SQLObject):
    username = StringCol(notNone=True)
    hostname = StringCol(notNone=True)
    message = UnicodeCol(notNone=True)
    timestamp = DateTimeCol(default=datetime.utcnow)
    
class Config(SQLObject):

    class sqlmeta:
        createSQL = "insert into config (config_key, config_value) values ('VERSION', '2')"
        
    config_key = StringCol(notNone=True)
    config_value = StringCol(notNone=True)
    
class UserGroups(SQLObject):
    hostname = StringCol(notNone=True)
    username = StringCol(notNone=True)
    groupname = StringCol(notNone=True)

class ClientComponent(SQLObject):
    hostname = StringCol(notNone=True)
    timestamp = DateTimeCol(default=datetime.utcnow)
    component = StringCol(notNone=True)
    value = StringCol(notNone=True)
    
class ClientActivity(SQLObject):
    hostname = StringCol(notNone=True)
    timestamp = DateTimeCol(default=datetime.utcnow)
    
class InstallQueue(SQLObject):
    hostname = StringCol(notNone=True)
    package = StringCol(notNone=True)
    # Return code of the install command
    status = IntCol(notNone=False, default=0)
    # Output of the installation process
    lastlog = StringCol(notNone=False)
    # Time the install has been scheduled
    inserttime = DateTimeCol(default=datetime.utcnow)
    # Time the installation result has been reported
    lastlog_time = DateTimeCol(notNone=False)

class Upgrade(SQLObject):
    name = StringCol(notNone=True, alternateID=True, alternateMethodName='by_name', unique=True)
    type = StringCol(notNone=True, default="upgrade")
    notes = UnicodeCol(notNone=True, default="")
    scheduled = DateTimeCol(notNone = False, default=None)
    completed = DateTimeCol(notNone = False, default=None)
    hosts = MultipleJoin("UpgradeHost", joinColumn="upgrade_id",orderBy="name")

class UpgradeHost(SQLObject):
    upgrade = ForeignKey("Upgrade", cascade = True, notNull = True)
    name = StringCol(notNone = True)
    started = DateTimeCol(notNone = False, default=None)
    ended = DateTimeCol(notNone = False, default=None)
    result = IntCol(notNone = False, default=None)
    output = UnicodeCol(notNone = False, default=None)

    idx_unique = index.DatabaseIndex(upgrade, name, unique=True)
    idx_name = index.DatabaseIndex(name)

class Scripts(SQLObject):
    name = StringCol(notNone=True, alternateID=True, alternateMethodName='by_name', unique=True)
    script = UnicodeCol(notNone=True, default="")
    runs = MultipleJoin("ScriptRun", joinColumn="run_id",orderBy="name")

class ScriptRun(SQLObject):
    name = StringCol(notNone=True, alternateID=True, alternateMethodName='by_name', unique=True)
    script = ForeignKey("Scripts", cascade = True, notNull = True)
    notes = UnicodeCol(notNone=True, default="")
    scheduled = DateTimeCol(notNone = False, default=None)
    completed = DateTimeCol(notNone = False, default=None)
    hosts = MultipleJoin("ScriptRunHost", joinColumn="script_run_id",orderBy="name")

class ScriptRunHost(SQLObject):
    script_run = ForeignKey("ScriptRun", cascade = True, notNull = True)
    name = StringCol(notNone = True)
    started = DateTimeCol(notNone = False, default=None)
    ended = DateTimeCol(notNone = False, default=None)
    result = IntCol(notNone = False, default=None)
    output = UnicodeCol(notNone = False, default=None)

    idx_unique = index.DatabaseIndex(script_run, name, unique=True)
    idx_name = index.DatabaseIndex(name)

class AuthPath(SQLObject):
    user_name = UnicodeCol(length=16)
    path = UnicodeCol(length=300)
    
class Auth(SQLObject):
    user_name = UnicodeCol(length=16, alternateID=True,
                           alternateMethodName="by_user_name")
    email_address = UnicodeCol(length=255, alternateID=True,
                               alternateMethodName="by_email_address")
    display_name = UnicodeCol(length=255)
    password = UnicodeCol(length=40)

    def _set_password(self, val):
        self._SO_set_password(crypt_password(val))

    def verify_password(self, val):
        return verify_password(val, self.password)

SALTCHARS="".join(map(chr, xrange(33,126)))

def crypt_password(pwd, salt=None):
    """Crypt a password.

    salt is randomly generated if it is not given"""
    if salt is None:
        salt = random.choice(SALTCHARS) + random.choice(SALTCHARS)
    h = Hasher()
    h.update(salt + pwd)
    return salt + h.hexdigest()

def verify_password(pwd, hash_pwd):
    """Verify a password against a hashed password"""
    salt = hash_pwd[:2]
    h = Hasher()
    h.update(salt + pwd)
    return salt + h.hexdigest() == hash_pwd


__all__ = [ClientActivity, InstallQueue, Users, Config, UserGroups, ClientComponent, Upgrade, UpgradeHost, Scripts, ScriptRun, ScriptRunHost, Auth, AuthPath, Request]

def init(conf):
    # Get the database filename from the configuration
    __DB = os.environ.get("OCTOFUSS_DB", None)
    if not __DB:
        __DB = conf.get("DB", "database")
    db_filename = os.path.abspath(__DB)

    # check if we need to upgrade

    _c = sqlite_connect(db_filename)
    _cur = _c.cursor()
    _cur.execute("select * from sqlite_master where type='table' and name='config'")
    if len(_cur.fetchall()) > 0:
        _cur.execute("select config_value from config where config_key = 'VERSION'")
        for row in _cur.fetchall():
            if str(row[0]) == "1":
                print "WE NEED AN UPGRADE, DO SOMETHING HERE"
                import shutil
                shutil.copyfile(db_filename, db_filename+".version1.bkp")

    connection_string = 'sqlite://' + db_filename
    connection = connectionForURI(connection_string)
    sqlhub.processConnection = connection

    for x in __all__:
      try:
        x.createTable()
      except:
        pass
    _cur.execute("create table if not exists octomon_log (tsend float not null primary key, tsstart float not null, lines integer not null)")
